from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_DLL, SDL_DLL

from .SDL_guid import SDL_GUID
from .SDL_power import SDL_PowerState
from .SDL_sensor import SDL_SensorType
from .SDL_iostream import SDL_IOStream
from .SDL_properties import SDL_PropertiesID
from .SDL_joystick import SDL_Joystick, SDL_JoystickID, SDL_JoystickConnectionState, \
    SDL_PROP_JOYSTICK_CAP_MONO_LED_BOOLEAN, SDL_PROP_JOYSTICK_CAP_RGB_LED_BOOLEAN, \
    SDL_PROP_JOYSTICK_CAP_PLAYER_LED_BOOLEAN, SDL_PROP_JOYSTICK_CAP_RUMBLE_BOOLEAN, \
    SDL_PROP_JOYSTICK_CAP_TRIGGER_RUMBLE_BOOLEAN

SDL_SET_CURRENT_DLL(SDL_DLL)

class SDL_Gamepad(ctypes.c_void_p):
    ...

SDL_GamepadType = ctypes.c_int

SDL_GAMEPAD_TYPE_UNKNOWN = 0
SDL_GAMEPAD_TYPE_STANDARD = 1
SDL_GAMEPAD_TYPE_XBOX360 = 2
SDL_GAMEPAD_TYPE_XBOXONE = 3
SDL_GAMEPAD_TYPE_PS3 = 4
SDL_GAMEPAD_TYPE_PS4 = 5
SDL_GAMEPAD_TYPE_PS5 = 6
SDL_GAMEPAD_TYPE_NINTENDO_SWITCH_PRO = 7
SDL_GAMEPAD_TYPE_NINTENDO_SWITCH_JOYCON_LEFT = 8
SDL_GAMEPAD_TYPE_NINTENDO_SWITCH_JOYCON_RIGHT = 9
SDL_GAMEPAD_TYPE_NINTENDO_SWITCH_JOYCON_PAIR = 10
SDL_GAMEPAD_TYPE_COUNT = 11

SDL_GamepadButton = ctypes.c_int

SDL_GAMEPAD_BUTTON_INVALID = -1
SDL_GAMEPAD_BUTTON_SOUTH = 0
SDL_GAMEPAD_BUTTON_EAST = 1
SDL_GAMEPAD_BUTTON_WEST = 2
SDL_GAMEPAD_BUTTON_NORTH = 3
SDL_GAMEPAD_BUTTON_BACK = 4
SDL_GAMEPAD_BUTTON_GUIDE = 5
SDL_GAMEPAD_BUTTON_START = 6
SDL_GAMEPAD_BUTTON_LEFT_STICK = 7
SDL_GAMEPAD_BUTTON_RIGHT_STICK = 8
SDL_GAMEPAD_BUTTON_LEFT_SHOULDER = 9
SDL_GAMEPAD_BUTTON_RIGHT_SHOULDER = 10
SDL_GAMEPAD_BUTTON_DPAD_UP = 11
SDL_GAMEPAD_BUTTON_DPAD_DOWN = 12
SDL_GAMEPAD_BUTTON_DPAD_LEFT = 13
SDL_GAMEPAD_BUTTON_DPAD_RIGHT = 14
SDL_GAMEPAD_BUTTON_MISC1 = 15
SDL_GAMEPAD_BUTTON_RIGHT_PADDLE1 = 16
SDL_GAMEPAD_BUTTON_LEFT_PADDLE1 = 17
SDL_GAMEPAD_BUTTON_RIGHT_PADDLE2 = 18
SDL_GAMEPAD_BUTTON_LEFT_PADDLE2 = 19
SDL_GAMEPAD_BUTTON_TOUCHPAD = 20
SDL_GAMEPAD_BUTTON_MISC2 = 21
SDL_GAMEPAD_BUTTON_MISC3 = 22
SDL_GAMEPAD_BUTTON_MISC4 = 23
SDL_GAMEPAD_BUTTON_MISC5 = 24
SDL_GAMEPAD_BUTTON_MISC6 = 25
SDL_GAMEPAD_BUTTON_COUNT = 26

SDL_GamepadButtonLabel = ctypes.c_int

SDL_GAMEPAD_BUTTON_LABEL_UNKNOWN = 0
SDL_GAMEPAD_BUTTON_LABEL_A = 1
SDL_GAMEPAD_BUTTON_LABEL_B = 2
SDL_GAMEPAD_BUTTON_LABEL_X = 3
SDL_GAMEPAD_BUTTON_LABEL_Y = 4
SDL_GAMEPAD_BUTTON_LABEL_CROSS = 5
SDL_GAMEPAD_BUTTON_LABEL_CIRCLE = 6
SDL_GAMEPAD_BUTTON_LABEL_SQUARE = 7
SDL_GAMEPAD_BUTTON_LABEL_TRIANGLE = 8

SDL_GamepadAxis = ctypes.c_int

SDL_GAMEPAD_AXIS_INVALID = -1
SDL_GAMEPAD_AXIS_LEFTX = 0
SDL_GAMEPAD_AXIS_LEFTY = 1
SDL_GAMEPAD_AXIS_RIGHTX = 2
SDL_GAMEPAD_AXIS_RIGHTY = 3
SDL_GAMEPAD_AXIS_LEFT_TRIGGER = 4
SDL_GAMEPAD_AXIS_RIGHT_TRIGGER = 5
SDL_GAMEPAD_AXIS_COUNT = 6

SDL_GamepadBindingType = ctypes.c_int

SDL_GAMEPAD_BINDTYPE_NONE = 0
SDL_GAMEPAD_BINDTYPE_BUTTON = 1
SDL_GAMEPAD_BINDTYPE_AXIS = 2
SDL_GAMEPAD_BINDTYPE_HAT = 3

class _axis(ctypes.Structure):
    _fields_ = [
        ("axis", SDL_GamepadAxis),
        ("axis_min", ctypes.c_int),
        ("axis_max", ctypes.c_int)
    ]

class _hat(ctypes.Structure):
    _fields_ = [
        ("hat", ctypes.c_int),
        ("mask", ctypes.c_int)
    ]

class _input(ctypes.Union):
    _fields_ = [
        ("button", SDL_GamepadButton),
        ("axis", _axis),
        ("hat", _hat)
    ]

class _output(ctypes.Union):
    _fields_ = [
        ("button", SDL_GamepadButton),
        ("axis", _axis)
    ]

class SDL_GamepadBinding(ctypes.Structure):
    _fields_ = [
        ("input_type", SDL_GamepadBindingType),
        ("input", _input),
        ("output_type", SDL_GamepadBindingType),
        ("output", _output)
    ]

SDL_FUNC("SDL_AddGamepadMapping", ctypes.c_int, ctypes.c_char_p)
SDL_FUNC("SDL_AddGamepadMappingsFromIO", ctypes.c_int, ctypes.POINTER(SDL_IOStream), ctypes.c_bool)
SDL_FUNC("SDL_AddGamepadMappingsFromFile", ctypes.c_int, ctypes.c_char_p)
SDL_FUNC("SDL_ReloadGamepadMappings", ctypes.c_bool)
SDL_FUNC("SDL_GetGamepadMappings", ctypes.POINTER(ctypes.c_char_p), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("SDL_GetGamepadMappingForGUID", ctypes.c_char_p, SDL_GUID)
SDL_FUNC("SDL_GetGamepadMapping", ctypes.c_char_p, ctypes.POINTER(SDL_Gamepad))
SDL_FUNC("SDL_SetGamepadMapping", ctypes.c_bool, SDL_JoystickID, ctypes.c_char_p)
SDL_FUNC("SDL_HasGamepad", ctypes.c_bool)
SDL_FUNC("SDL_GetGamepads", ctypes.POINTER(SDL_JoystickID), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("SDL_IsGamepad", ctypes.c_bool, SDL_JoystickID)
SDL_FUNC("SDL_GetGamepadNameForID", ctypes.c_char_p, SDL_JoystickID)
SDL_FUNC("SDL_GetGamepadPathForID", ctypes.c_char_p, SDL_JoystickID)
SDL_FUNC("SDL_GetGamepadPlayerIndexForID", ctypes.c_int, SDL_JoystickID)
SDL_FUNC("SDL_GetGamepadGUIDForID", SDL_GUID, SDL_JoystickID)
SDL_FUNC("SDL_GetGamepadVendorForID", ctypes.c_uint16, SDL_JoystickID)
SDL_FUNC("SDL_GetGamepadProductForID", ctypes.c_uint16, SDL_JoystickID)
SDL_FUNC("SDL_GetGamepadProductVersionForID", ctypes.c_uint16, SDL_JoystickID)
SDL_FUNC("SDL_GetGamepadTypeForID", SDL_GamepadType, SDL_JoystickID)
SDL_FUNC("SDL_GetRealGamepadTypeForID", SDL_GamepadType, SDL_JoystickID)
SDL_FUNC("SDL_GetGamepadMappingForID", ctypes.c_char_p, SDL_JoystickID)
SDL_FUNC("SDL_OpenGamepad", ctypes.POINTER(SDL_Gamepad), SDL_JoystickID)
SDL_FUNC("SDL_GetGamepadFromID", ctypes.POINTER(SDL_Gamepad), SDL_JoystickID)
SDL_FUNC("SDL_GetGamepadFromPlayerIndex", ctypes.POINTER(SDL_Gamepad), ctypes.c_int)
SDL_FUNC("SDL_GetGamepadProperties", SDL_PropertiesID, ctypes.POINTER(SDL_Gamepad))

SDL_PROP_GAMEPAD_CAP_MONO_LED_BOOLEAN = SDL_PROP_JOYSTICK_CAP_MONO_LED_BOOLEAN
SDL_PROP_GAMEPAD_CAP_RGB_LED_BOOLEAN = SDL_PROP_JOYSTICK_CAP_RGB_LED_BOOLEAN
SDL_PROP_GAMEPAD_CAP_PLAYER_LED_BOOLEAN = SDL_PROP_JOYSTICK_CAP_PLAYER_LED_BOOLEAN
SDL_PROP_GAMEPAD_CAP_RUMBLE_BOOLEAN = SDL_PROP_JOYSTICK_CAP_RUMBLE_BOOLEAN
SDL_PROP_GAMEPAD_CAP_TRIGGER_RUMBLE_BOOLEAN = SDL_PROP_JOYSTICK_CAP_TRIGGER_RUMBLE_BOOLEAN

SDL_FUNC("SDL_GetGamepadID", SDL_JoystickID, ctypes.POINTER(SDL_Gamepad))
SDL_FUNC("SDL_GetGamepadName", ctypes.c_char_p, ctypes.POINTER(SDL_Gamepad))
SDL_FUNC("SDL_GetGamepadPath", ctypes.c_char_p, ctypes.POINTER(SDL_Gamepad))
SDL_FUNC("SDL_GetGamepadType", SDL_GamepadType, ctypes.POINTER(SDL_Gamepad))
SDL_FUNC("SDL_GetRealGamepadType", SDL_GamepadType, ctypes.POINTER(SDL_Gamepad))
SDL_FUNC("SDL_GetGamepadPlayerIndex", ctypes.c_int, ctypes.POINTER(SDL_Gamepad))
SDL_FUNC("SDL_SetGamepadPlayerIndex", ctypes.c_bool, ctypes.POINTER(SDL_Gamepad), ctypes.c_int)
SDL_FUNC("SDL_GetGamepadVendor", ctypes.c_uint16, ctypes.POINTER(SDL_Gamepad))
SDL_FUNC("SDL_GetGamepadProduct", ctypes.c_uint16, ctypes.POINTER(SDL_Gamepad))
SDL_FUNC("SDL_GetGamepadProductVersion", ctypes.c_uint16, ctypes.POINTER(SDL_Gamepad))
SDL_FUNC("SDL_GetGamepadFirmwareVersion", ctypes.c_uint16, ctypes.POINTER(SDL_Gamepad))
SDL_FUNC("SDL_GetGamepadSerial", ctypes.c_char_p, ctypes.POINTER(SDL_Gamepad))
SDL_FUNC("SDL_GetGamepadSteamHandle", ctypes.c_uint64, ctypes.POINTER(SDL_Gamepad))
SDL_FUNC("SDL_GetGamepadConnectionState", SDL_JoystickConnectionState, ctypes.POINTER(SDL_Gamepad))
SDL_FUNC("SDL_GetGamepadPowerInfo", SDL_PowerState, ctypes.POINTER(SDL_Gamepad), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("SDL_GamepadConnected", ctypes.c_bool, ctypes.POINTER(SDL_Gamepad))
SDL_FUNC("SDL_GetGamepadJoystick", ctypes.POINTER(SDL_Joystick), ctypes.POINTER(SDL_Gamepad))
SDL_FUNC("SDL_SetGamepadEventsEnabled", None, ctypes.c_bool)
SDL_FUNC("SDL_GamepadEventsEnabled", ctypes.c_bool)
SDL_FUNC("SDL_GetGamepadBindings", ctypes.POINTER(ctypes.POINTER(SDL_GamepadBinding)), ctypes.POINTER(SDL_Gamepad), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("SDL_UpdateGamepads", None)
SDL_FUNC("SDL_GetGamepadTypeFromString", SDL_GamepadType, ctypes.c_char_p)
SDL_FUNC("SDL_GetGamepadStringForType", ctypes.c_char_p, SDL_GamepadType)
SDL_FUNC("SDL_GetGamepadAxisFromString", SDL_GamepadAxis, ctypes.c_char_p)
SDL_FUNC("SDL_GetGamepadStringForAxis", ctypes.c_char_p, SDL_GamepadAxis)
SDL_FUNC("SDL_GamepadHasAxis", ctypes.c_bool, ctypes.POINTER(SDL_Gamepad), SDL_GamepadAxis)
SDL_FUNC("SDL_GetGamepadAxis", ctypes.c_int16, ctypes.POINTER(SDL_Gamepad), SDL_GamepadAxis)
SDL_FUNC("SDL_GetGamepadButtonFromString", SDL_GamepadButton, ctypes.c_char_p)
SDL_FUNC("SDL_GetGamepadStringForButton", ctypes.c_char_p, SDL_GamepadButton)
SDL_FUNC("SDL_GamepadHasButton", ctypes.c_bool, ctypes.POINTER(SDL_Gamepad), SDL_GamepadButton)
SDL_FUNC("SDL_GetGamepadButton", ctypes.c_bool, ctypes.POINTER(SDL_Gamepad), SDL_GamepadButton)
SDL_FUNC("SDL_GetGamepadButtonLabelForType", SDL_GamepadButtonLabel, SDL_GamepadType, SDL_GamepadButton)
SDL_FUNC("SDL_GetGamepadButtonLabel", SDL_GamepadButtonLabel, ctypes.POINTER(SDL_Gamepad), SDL_GamepadButton)
SDL_FUNC("SDL_GetNumGamepadTouchpads", ctypes.c_int, ctypes.POINTER(SDL_Gamepad))
SDL_FUNC("SDL_GetNumGamepadTouchpadFingers", ctypes.c_int, ctypes.POINTER(SDL_Gamepad), ctypes.c_int)
SDL_FUNC("SDL_GetGamepadTouchpadFinger", ctypes.c_bool, ctypes.POINTER(SDL_Gamepad), ctypes.c_int, ctypes.c_int, ctypes.POINTER(ctypes.c_uint8), ctypes.POINTER(ctypes.c_float), ctypes.POINTER(ctypes.c_float), ctypes.POINTER(ctypes.c_float))
SDL_FUNC("SDL_GamepadHasSensor", ctypes.c_bool, ctypes.POINTER(SDL_Gamepad), SDL_SensorType)
SDL_FUNC("SDL_SetGamepadSensorEnabled", ctypes.c_bool, ctypes.POINTER(SDL_Gamepad), SDL_SensorType, ctypes.c_bool)
SDL_FUNC("SDL_GamepadSensorEnabled", ctypes.c_bool, ctypes.POINTER(SDL_Gamepad), SDL_SensorType)
SDL_FUNC("SDL_GetGamepadSensorDataRate", ctypes.c_float, ctypes.POINTER(SDL_Gamepad), SDL_SensorType)
SDL_FUNC("SDL_GetGamepadSensorData", ctypes.c_bool, ctypes.POINTER(SDL_Gamepad), SDL_SensorType, ctypes.POINTER(ctypes.c_float), ctypes.c_int)
SDL_FUNC("SDL_RumbleGamepad", ctypes.c_bool, ctypes.POINTER(SDL_Gamepad), ctypes.c_uint16, ctypes.c_uint16, ctypes.c_uint32)
SDL_FUNC("SDL_RumbleGamepadTriggers", ctypes.c_bool, ctypes.POINTER(SDL_Gamepad), ctypes.c_uint16, ctypes.c_uint16, ctypes.c_uint32)
SDL_FUNC("SDL_SetGamepadLED", ctypes.c_bool, ctypes.POINTER(SDL_Gamepad), ctypes.c_uint8, ctypes.c_uint8, ctypes.c_uint8)
SDL_FUNC("SDL_SendGamepadEffect", ctypes.c_bool, ctypes.POINTER(SDL_Gamepad), ctypes.c_void_p, ctypes.c_int)
SDL_FUNC("SDL_CloseGamepad", None, ctypes.POINTER(SDL_Gamepad))

SDL_FUNC("SDL_GetGamepadAppleSFSymbolsNameForButton", ctypes.c_char_p, ctypes.POINTER(SDL_Gamepad), SDL_GamepadButton)
SDL_FUNC("SDL_GetGamepadAppleSFSymbolsNameForAxis", ctypes.c_char_p, ctypes.POINTER(SDL_Gamepad), SDL_GamepadAxis)


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_DLL, SDL_DLL

SDL_SET_CURRENT_DLL(SDL_DLL)

def SDL_HasExactlyOneBitSet32(x: ctypes.c_uint32) -> ctypes.c_bool:
    if x and not (x & (x - 1)): return ctypes.c_bool(1)
    else: return ctypes.c_bool(0)


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_DLL, SDL_DLL

from .SDL_filesystem import SDL_EnumerateDirectoryCallback, SDL_PathInfo, SDL_GlobFlags
from .SDL_properties import SDL_PropertiesID

SDL_SET_CURRENT_DLL(SDL_DLL)

class SDL_StorageInterface(ctypes.Structure):
    _fields_ = [
        ("version", ctypes.c_uint32),
        ("close", ctypes.CFUNCTYPE(ctypes.c_bool, ctypes.c_void_p)),
        ("ready", ctypes.CFUNCTYPE(ctypes.c_bool, ctypes.c_void_p)),
        ("enumerate", ctypes.CFUNCTYPE(ctypes.c_bool, ctypes.c_void_p, ctypes.c_char_p, SDL_EnumerateDirectoryCallback, ctypes.c_void_p)),
        ("info", ctypes.CFUNCTYPE(ctypes.c_bool, ctypes.c_void_p, ctypes.c_char_p, ctypes.POINTER(SDL_PathInfo))),
        ("read_file", ctypes.CFUNCTYPE(ctypes.c_bool, ctypes.c_void_p, ctypes.c_char_p, ctypes.c_void_p, ctypes.c_uint64)),
        ("write_file", ctypes.CFUNCTYPE(ctypes.c_bool, ctypes.c_void_p, ctypes.c_char_p, ctypes.c_void_p, ctypes.c_uint64)),
        ("mkdir", ctypes.CFUNCTYPE(ctypes.c_bool, ctypes.c_void_p, ctypes.c_char_p)),
        ("remove", ctypes.CFUNCTYPE(ctypes.c_bool, ctypes.c_void_p, ctypes.c_char_p)),
        ("rename", ctypes.CFUNCTYPE(ctypes.c_bool, ctypes.c_void_p, ctypes.c_char_p, ctypes.c_char_p)),
        ("copy", ctypes.CFUNCTYPE(ctypes.c_bool, ctypes.c_void_p, ctypes.c_char_p, ctypes.c_char_p)),
        ("space_remaining", ctypes.CFUNCTYPE(ctypes.c_uint64, ctypes.c_void_p))
    ]

class SDL_Storage(ctypes.c_void_p):
    ...

SDL_FUNC("SDL_OpenTitleStorage", ctypes.POINTER(SDL_Storage), ctypes.c_char_p, SDL_PropertiesID)
SDL_FUNC("SDL_OpenUserStorage", ctypes.POINTER(SDL_Storage), ctypes.c_char_p, ctypes.c_char_p, SDL_PropertiesID)
SDL_FUNC("SDL_OpenFileStorage", ctypes.POINTER(SDL_Storage), ctypes.c_char_p, SDL_PropertiesID)
SDL_FUNC("SDL_OpenStorage", ctypes.POINTER(SDL_Storage), ctypes.POINTER(SDL_StorageInterface), ctypes.c_void_p)
SDL_FUNC("SDL_CloseStorage", ctypes.c_bool, ctypes.POINTER(SDL_Storage))
SDL_FUNC("SDL_StorageReady", ctypes.c_bool, ctypes.POINTER(SDL_Storage))
SDL_FUNC("SDL_GetStorageFileSize", ctypes.c_bool, ctypes.POINTER(SDL_Storage), ctypes.c_char_p, ctypes.POINTER(ctypes.c_uint64))
SDL_FUNC("SDL_ReadStorageFile", ctypes.c_bool, ctypes.POINTER(SDL_Storage), ctypes.c_char_p, ctypes.c_void_p, ctypes.c_uint64)
SDL_FUNC("SDL_WriteStorageFile", ctypes.c_bool, ctypes.POINTER(SDL_Storage), ctypes.c_char_p, ctypes.c_void_p, ctypes.c_uint64)
SDL_FUNC("SDL_CreateStorageDirectory", ctypes.c_bool, ctypes.POINTER(SDL_Storage), ctypes.c_char_p)
SDL_FUNC("SDL_EnumerateStorageDirectory", ctypes.c_bool, ctypes.POINTER(SDL_Storage), ctypes.c_char_p, SDL_EnumerateDirectoryCallback, ctypes.c_void_p)
SDL_FUNC("SDL_RemoveStoragePath", ctypes.c_bool, ctypes.POINTER(SDL_Storage), ctypes.c_char_p)
SDL_FUNC("SDL_RenameStoragePath", ctypes.c_bool, ctypes.POINTER(SDL_Storage), ctypes.c_char_p, ctypes.c_char_p)
SDL_FUNC("SDL_CopyStorageFile", ctypes.c_bool, ctypes.POINTER(SDL_Storage), ctypes.c_char_p, ctypes.c_char_p)
SDL_FUNC("SDL_GetStoragePathInfo", ctypes.c_bool, ctypes.POINTER(SDL_Storage), ctypes.c_char_p, ctypes.POINTER(SDL_PathInfo))
SDL_FUNC("SDL_GetStorageSpaceRemaining", ctypes.c_uint64, ctypes.POINTER(SDL_Storage))
SDL_FUNC("SDL_GlobStorageDirectory", ctypes.POINTER(ctypes.c_char_p), ctypes.POINTER(SDL_Storage), ctypes.c_char_p, ctypes.c_char_p, SDL_GlobFlags, ctypes.POINTER(ctypes.c_int))


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_DLL, SDL_DLL

from .SDL_stdinc import SDL_Time

SDL_SET_CURRENT_DLL(SDL_DLL)

class SDL_DateTime(ctypes.Structure):
    _fields_ = [
        ("year", ctypes.c_int),
        ("month", ctypes.c_int),
        ("day", ctypes.c_int),
        ("hour", ctypes.c_int),
        ("minute", ctypes.c_int),
        ("second", ctypes.c_int),
        ("nanosecond", ctypes.c_int),
        ("day_of_week", ctypes.c_int),
        ("utc_offset", ctypes.c_int)
    ]

SDL_DateFormat = ctypes.c_int

SDL_DATE_FORMAT_YYYYMMDD = 0
SDL_DATE_FORMAT_DDMMYYYY = 1
SDL_DATE_FORMAT_MMDDYYYY = 2 

SDL_TimeFormat = ctypes.c_int

SDL_TIME_FORMAT_24HR = 0
SDL_TIME_FORMAT_12HR = 1

SDL_FUNC("SDL_GetDateTimeLocalePreferences", ctypes.c_bool, ctypes.POINTER(SDL_DateFormat), ctypes.POINTER(SDL_TimeFormat))
SDL_FUNC("SDL_GetCurrentTime", ctypes.c_bool, ctypes.POINTER(SDL_Time))
SDL_FUNC("SDL_TimeToDateTime", ctypes.c_bool, SDL_Time, ctypes.POINTER(SDL_DateTime), ctypes.c_bool)
SDL_FUNC("SDL_DateTimeToTime", ctypes.c_bool, ctypes.POINTER(SDL_DateTime), ctypes.POINTER(SDL_Time))
SDL_FUNC("SDL_TimeToWindows", None, SDL_Time, ctypes.POINTER(ctypes.c_uint32), ctypes.POINTER(ctypes.c_uint32))
SDL_FUNC("SDL_TimeFromWindows", SDL_Time, ctypes.c_uint32, ctypes.c_uint32)
SDL_FUNC("SDL_GetDaysInMonth", ctypes.c_int, ctypes.c_int, ctypes.c_int)
SDL_FUNC("SDL_GetDayOfYear", ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int)
SDL_FUNC("SDL_GetDayOfWeek", ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int)


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_DLL, SDL_DLL

from .SDL_joystick import SDL_Joystick

SDL_SET_CURRENT_DLL(SDL_DLL)

class SDL_Haptic(ctypes.c_void_p):
    ...

SDL_HAPTIC_CONSTANT = 1 << 0
SDL_HAPTIC_SINE = 1 << 1
SDL_HAPTIC_SQUARE = 1 << 2
SDL_HAPTIC_TRIANGLE = 1 << 3
SDL_HAPTIC_SAWTOOTHUP = 1 << 4
SDL_HAPTIC_SAWTOOTHDOWN = 1 << 5
SDL_HAPTIC_RAMP = 1 << 6
SDL_HAPTIC_SPRING = 1 << 7
SDL_HAPTIC_DAMPER = 1 << 8
SDL_HAPTIC_INERTIA = 1 << 9
SDL_HAPTIC_FRICTION = 1 << 10
SDL_HAPTIC_LEFTRIGHT = 1 << 11
SDL_HAPTIC_RESERVED1 = 1 << 12
SDL_HAPTIC_RESERVED2 = 1 << 13
SDL_HAPTIC_RESERVED3 = 1 << 14
SDL_HAPTIC_CUSTOM = 1 << 15
SDL_HAPTIC_GAIN = 1 << 16
SDL_HAPTIC_AUTOCENTER = 1 << 17
SDL_HAPTIC_STATUS = 1 << 18
SDL_HAPTIC_PAUSE = 1 << 19

SDL_HAPTIC_POLAR = 0
SDL_HAPTIC_CARTESIAN = 1
SDL_HAPTIC_SPHERICAL = 2
SDL_HAPTIC_STEERING_AXIS = 3

SDL_HAPTIC_INFINITY = 4294967295

class SDL_HapticDirection(ctypes.Structure):
    _fields_ = [
        ("type", ctypes.c_uint8),
        ("dir", ctypes.c_int32* 3)
    ]

class SDL_HapticConstant(ctypes.Structure):
    _fields_ = [
        ("type", ctypes.c_uint16),
        ("direction", SDL_HapticDirection),
        ("length", ctypes.c_uint32),
        ("delay", ctypes.c_uint16),
        ("button", ctypes.c_uint16),
        ("interval", ctypes.c_uint16),
        ("level", ctypes.c_int16),
        ("attack_length", ctypes.c_uint16),
        ("attack_level", ctypes.c_uint16),
        ("fade_length", ctypes.c_uint16),
        ("fade_level", ctypes.c_uint16)
    ]

class SDL_HapticPeriodic(ctypes.Structure):
    _fields_ = [
        ("type", ctypes.c_uint16),
        ("direction", SDL_HapticDirection),
        ("length", ctypes.c_uint32),
        ("delay", ctypes.c_uint16),
        ("button", ctypes.c_uint16),
        ("interval", ctypes.c_uint16),
        ("period", ctypes.c_uint16),
        ("magnitude", ctypes.c_int16),
        ("offset", ctypes.c_int16),
        ("phase", ctypes.c_uint16),
        ("attack_length", ctypes.c_uint16),
        ("attack_level", ctypes.c_uint16),
        ("fade_length", ctypes.c_uint16),
        ("fade_level", ctypes.c_uint16)
    ]

class SDL_HapticCondition(ctypes.Structure):
    _fields_ = [
        ("type", ctypes.c_uint16),
        ("direction", SDL_HapticDirection),
        ("length", ctypes.c_uint32),
        ("delay", ctypes.c_uint16),
        ("button", ctypes.c_uint16),
        ("interval", ctypes.c_uint16),
        ("right_sat", ctypes.c_uint16* 3),
        ("left_sat", ctypes.c_uint16* 3),
        ("right_coeff", ctypes.c_int16* 3),
        ("left_coeff", ctypes.c_int16* 3),
        ("deadband", ctypes.c_uint16* 3),
        ("center", ctypes.c_int16* 3)
    ]

class SDL_HapticRamp(ctypes.Structure):
    _fields_ = [
        ("type", ctypes.c_uint16),
        ("direction", SDL_HapticDirection),
        ("length", ctypes.c_uint32),
        ("delay", ctypes.c_uint16),
        ("button", ctypes.c_uint16),
        ("interval", ctypes.c_uint16),
        ("start", ctypes.c_int16),
        ("end", ctypes.c_int16),
        ("attack_length", ctypes.c_uint16),
        ("attack_level", ctypes.c_uint16),
        ("fade_length", ctypes.c_uint16),
        ("fade_level", ctypes.c_uint16)
    ]

class SDL_HapticLeftRight(ctypes.Structure):
    _fields_ = [
        ("type", ctypes.c_uint16),
        ("length", ctypes.c_uint32),
        ("large_magnitude", ctypes.c_uint16),
        ("small_magnitude", ctypes.c_uint16)
    ]

class SDL_HapticCustom(ctypes.Structure):
    _fields_ = [
        ("type", ctypes.c_uint16),
        ("direction", SDL_HapticDirection),
        ("length", ctypes.c_uint32),
        ("delay", ctypes.c_uint16),
        ("button", ctypes.c_uint16),
        ("interval", ctypes.c_uint16),
        ("channels", ctypes.c_uint8),
        ("period", ctypes.c_uint16),
        ("samples", ctypes.c_uint16),
        ("data", ctypes.POINTER(ctypes.c_uint16)),
        ("attack_length", ctypes.c_uint16),
        ("attack_level", ctypes.c_uint16),
        ("fade_length", ctypes.c_uint16),
        ("fade_level", ctypes.c_uint16)
    ]

class SDL_HapticEffect(ctypes.Union):
    _fields_ = [
        ("type", ctypes.c_uint16),
        ("constant", SDL_HapticConstant),
        ("periodic", SDL_HapticPeriodic),
        ("condition", SDL_HapticCondition),
        ("ramp", SDL_HapticRamp),
        ("leftright", SDL_HapticLeftRight),
        ("custom", SDL_HapticCustom)
    ]

SDL_HapticID = ctypes.c_uint32

SDL_FUNC("SDL_GetHaptics", ctypes.POINTER(SDL_HapticID), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("SDL_GetHapticNameForID", ctypes.c_char_p, SDL_HapticID)
SDL_FUNC("SDL_OpenHaptic", ctypes.POINTER(SDL_Haptic), SDL_HapticID)
SDL_FUNC("SDL_GetHapticFromID", ctypes.POINTER(SDL_Haptic), SDL_HapticID)
SDL_FUNC("SDL_GetHapticID", SDL_HapticID, ctypes.POINTER(SDL_Haptic))
SDL_FUNC("SDL_GetHapticName", ctypes.c_char_p, ctypes.POINTER(SDL_Haptic))
SDL_FUNC("SDL_IsMouseHaptic", ctypes.c_bool)
SDL_FUNC("SDL_OpenHapticFromMouse", ctypes.POINTER(SDL_Haptic))
SDL_FUNC("SDL_IsJoystickHaptic", ctypes.c_bool, ctypes.POINTER(SDL_Joystick))
SDL_FUNC("SDL_OpenHapticFromJoystick", ctypes.POINTER(SDL_Haptic), ctypes.POINTER(SDL_Joystick))
SDL_FUNC("SDL_CloseHaptic", None, ctypes.POINTER(SDL_Haptic))
SDL_FUNC("SDL_GetMaxHapticEffects", ctypes.c_int, ctypes.POINTER(SDL_Haptic))
SDL_FUNC("SDL_GetMaxHapticEffectsPlaying", ctypes.c_int, ctypes.POINTER(SDL_Haptic))
SDL_FUNC("SDL_GetHapticFeatures", ctypes.c_uint32, ctypes.POINTER(SDL_Haptic))
SDL_FUNC("SDL_GetNumHapticAxes", ctypes.c_int, ctypes.POINTER(SDL_Haptic))
SDL_FUNC("SDL_HapticEffectSupported", ctypes.c_bool, ctypes.POINTER(SDL_Haptic), ctypes.POINTER(SDL_HapticEffect))
SDL_FUNC("SDL_CreateHapticEffect", ctypes.c_int, ctypes.POINTER(SDL_Haptic), ctypes.POINTER(SDL_HapticEffect))
SDL_FUNC("SDL_UpdateHapticEffect", ctypes.c_bool, ctypes.POINTER(SDL_Haptic), ctypes.c_int, ctypes.POINTER(SDL_HapticEffect))
SDL_FUNC("SDL_RunHapticEffect", ctypes.c_bool, ctypes.POINTER(SDL_Haptic), ctypes.c_int, ctypes.c_uint32)
SDL_FUNC("SDL_StopHapticEffect", ctypes.c_bool, ctypes.POINTER(SDL_Haptic), ctypes.c_int)
SDL_FUNC("SDL_DestroyHapticEffect", None, ctypes.POINTER(SDL_Haptic), ctypes.c_int)
SDL_FUNC("SDL_GetHapticEffectStatus", ctypes.c_bool, ctypes.POINTER(SDL_Haptic), ctypes.c_int)
SDL_FUNC("SDL_SetHapticGain", ctypes.c_bool, ctypes.POINTER(SDL_Haptic), ctypes.c_int)
SDL_FUNC("SDL_SetHapticAutocenter", ctypes.c_bool, ctypes.POINTER(SDL_Haptic), ctypes.c_int)
SDL_FUNC("SDL_PauseHaptic", ctypes.c_bool, ctypes.POINTER(SDL_Haptic))
SDL_FUNC("SDL_ResumeHaptic", ctypes.c_bool, ctypes.POINTER(SDL_Haptic))
SDL_FUNC("SDL_StopHapticEffects", ctypes.c_bool, ctypes.POINTER(SDL_Haptic))
SDL_FUNC("SDL_HapticRumbleSupported", ctypes.c_bool, ctypes.POINTER(SDL_Haptic))
SDL_FUNC("SDL_InitHapticRumble", ctypes.c_bool, ctypes.POINTER(SDL_Haptic))
SDL_FUNC("SDL_PlayHapticRumble", ctypes.c_bool, ctypes.POINTER(SDL_Haptic), ctypes.c_float, ctypes.c_uint32)
SDL_FUNC("SDL_StopHapticRumble", ctypes.c_bool, ctypes.POINTER(SDL_Haptic))


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_DLL, SDL_DLL

SDL_SET_CURRENT_DLL(SDL_DLL)

class SDL_GUID(ctypes.Structure):
    _fields_ = [
        ("data", ctypes.c_uint8 * 16)
    ]

SDL_FUNC("SDL_GUIDToString", None, SDL_GUID, ctypes.c_char_p, ctypes.c_int)
SDL_FUNC("SDL_StringToGUID", SDL_GUID, ctypes.c_char_p)


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_DLL, SDL_DLL

from .SDL_pixels import SDL_PixelFormat, SDL_Colorspace
from .SDL_properties import SDL_PropertiesID
from .SDL_surface import SDL_Surface

SDL_SET_CURRENT_DLL(SDL_DLL)

SDL_CameraID = ctypes.c_uint32

class SDL_Camera(ctypes.c_void_p):
    ...

class SDL_CameraSpec(ctypes.Structure):
    _fields_ = [
        ("format", ctypes.POINTER(SDL_PixelFormat)),
        ("colorspace", SDL_Colorspace),
        ("width", ctypes.c_int),
        ("height", ctypes.c_int),
        ("framerate_numerator", ctypes.c_int),
        ("framerate_denominator", ctypes.c_int)
    ]

SDL_CameraPosition = ctypes.c_int

SDL_CAMERA_POSITION_UNKNOWN = 0
SDL_CAMERA_POSITION_FRONT_FACING = 1
SDL_CAMERA_POSITION_BACK_FACING = 2

SDL_FUNC("SDL_GetNumCameraDrivers", ctypes.c_int)
SDL_FUNC("SDL_GetCameraDriver", ctypes.c_char_p, ctypes.c_int)
SDL_FUNC("SDL_GetCurrentCameraDriver", ctypes.c_char_p)
SDL_FUNC("SDL_GetCameras", ctypes.POINTER(SDL_CameraID), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("SDL_GetCameraSupportedFormats", ctypes.POINTER(ctypes.POINTER(SDL_CameraSpec)), SDL_CameraID, ctypes.POINTER(ctypes.c_int))
SDL_FUNC("SDL_GetCameraName", ctypes.c_char_p, SDL_CameraID)
SDL_FUNC("SDL_GetCameraPosition", SDL_CameraPosition, SDL_CameraID)
SDL_FUNC("SDL_OpenCamera", ctypes.POINTER(SDL_Camera), SDL_CameraID, ctypes.POINTER(SDL_CameraSpec))
SDL_FUNC("SDL_GetCameraPermissionState", ctypes.c_int, ctypes.POINTER(SDL_Camera))
SDL_FUNC("SDL_GetCameraID", SDL_CameraID, ctypes.POINTER(SDL_Camera))
SDL_FUNC("SDL_GetCameraProperties", SDL_PropertiesID, ctypes.POINTER(SDL_Camera))
SDL_FUNC("SDL_GetCameraFormat", ctypes.c_bool, ctypes.POINTER(SDL_Camera), ctypes.POINTER(SDL_CameraSpec))
SDL_FUNC("SDL_AcquireCameraFrame", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_Camera), ctypes.POINTER(ctypes.c_int64))
SDL_FUNC("SDL_ReleaseCameraFrame", None, ctypes.POINTER(SDL_Camera), ctypes.POINTER(SDL_Surface))
SDL_FUNC("SDL_CloseCamera", None, ctypes.POINTER(SDL_Camera))


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_DLL, SDL_GET_DLL, SDL_NET_DLL, SDL_DLL

from .SDL_version import SDL_VERSIONNUM

SDL_SET_CURRENT_DLL(SDL_NET_DLL)

SDL_NET_MAJOR_VERSION = 3
SDL_NET_MINOR_VERSION = 0
SDL_NET_MICRO_VERSION = 0

SDL_NET_VERSION = \
    SDL_VERSIONNUM(SDL_NET_MAJOR_VERSION, SDL_NET_MINOR_VERSION, SDL_NET_MICRO_VERSION)

SDL_NET_VERSION_ATLEAST = lambda x, y, z: \
    (SDL_NET_MAJOR_VERSION >= x) and \
    (SDL_NET_MAJOR_VERSION > x or SDL_NET_MINOR_VERSION >= y) and \
    (SDL_NET_MAJOR_VERSION > x or SDL_NET_MINOR_VERSION > y or SDL_NET_MICRO_VERSION >= z)

SDL_FUNC("SDLNet_Version", ctypes.c_int)
SDL_FUNC("SDLNet_Init", ctypes.c_bool)
SDL_FUNC("SDLNet_Quit", None)

class SDLNet_Address(ctypes.c_void_p):
    ...

SDL_FUNC("SDLNet_ResolveHostname", ctypes.POINTER(SDLNet_Address), ctypes.c_char_p)
SDL_FUNC("SDLNet_WaitUntilResolved", ctypes.c_int, ctypes.POINTER(SDLNet_Address), ctypes.c_int32)
SDL_FUNC("SDLNet_GetAddressStatus", ctypes.c_int, ctypes.POINTER(SDLNet_Address))
SDL_FUNC("SDLNet_GetAddressString", ctypes.c_char_p, ctypes.POINTER(SDLNet_Address))
SDL_FUNC("SDLNet_RefAddress", ctypes.POINTER(SDLNet_Address), ctypes.POINTER(SDLNet_Address))
SDL_FUNC("SDLNet_UnrefAddress", None, ctypes.POINTER(SDLNet_Address))
SDL_FUNC("SDLNet_SimulateAddressResolutionLoss", None, ctypes.c_int)
SDL_FUNC("SDLNet_CompareAddresses", ctypes.c_int, ctypes.POINTER(SDLNet_Address), ctypes.POINTER(SDLNet_Address))
SDL_FUNC("SDLNet_GetLocalAddresses", ctypes.POINTER(ctypes.POINTER(SDLNet_Address)), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("SDLNet_FreeLocalAddresses", None, ctypes.POINTER(ctypes.POINTER(SDLNet_Address)))

class SDLNet_StreamSocket(ctypes.c_void_p):
    ...

SDL_FUNC("SDLNet_CreateClient", ctypes.POINTER(SDLNet_StreamSocket), ctypes.POINTER(SDLNet_Address), ctypes.c_uint16)
SDL_FUNC("SDLNet_WaitUntilConnected", ctypes.c_int, ctypes.POINTER(SDLNet_StreamSocket), ctypes.c_int32)

class SDLNet_Server(ctypes.c_void_p):
    ...

SDL_FUNC("SDLNet_CreateServer", ctypes.POINTER(SDLNet_Server), ctypes.POINTER(SDLNet_Address), ctypes.c_uint16)
SDL_FUNC("SDLNet_AcceptClient", ctypes.c_bool, ctypes.POINTER(SDLNet_Server), ctypes.POINTER(ctypes.POINTER(SDLNet_StreamSocket)))
SDL_FUNC("SDLNet_DestroyServer", None, ctypes.POINTER(SDLNet_Server))
SDL_FUNC("SDLNet_GetStreamSocketAddress", ctypes.POINTER(SDLNet_Address), ctypes.POINTER(SDLNet_StreamSocket))
SDL_FUNC("SDLNet_GetConnectionStatus", ctypes.c_int, ctypes.POINTER(SDLNet_StreamSocket))
SDL_FUNC("SDLNet_WriteToStreamSocket", ctypes.c_bool, ctypes.POINTER(SDLNet_StreamSocket), ctypes.c_void_p, ctypes.c_int)
SDL_FUNC("SDLNet_GetStreamSocketPendingWrites", ctypes.c_int, ctypes.POINTER(SDLNet_StreamSocket))
SDL_FUNC("SDLNet_WaitUntilStreamSocketDrained", ctypes.c_int, ctypes.POINTER(SDLNet_StreamSocket), ctypes.c_int32)
SDL_FUNC("SDLNet_ReadFromStreamSocket", ctypes.c_int, ctypes.POINTER(SDLNet_StreamSocket), ctypes.c_void_p, ctypes.c_int)
SDL_FUNC("SDLNet_SimulateStreamPacketLoss", None, ctypes.POINTER(SDLNet_StreamSocket), ctypes.c_int)
SDL_FUNC("SDLNet_DestroyStreamSocket", None, ctypes.POINTER(SDLNet_StreamSocket))

class SDLNet_DatagramSocket(ctypes.c_void_p):
    ...

class SDLNet_Datagram(ctypes.Structure):
    _fields_ = [
        ("addr", ctypes.POINTER(SDLNet_Address)),
        ("port", ctypes.c_uint16),
        ("buf", ctypes.POINTER(ctypes.c_uint8)),
        ("buflen", ctypes.c_int)
    ]

SDL_FUNC("SDLNet_CreateDatagramSocket", ctypes.POINTER(SDLNet_DatagramSocket), ctypes.POINTER(SDLNet_Address), ctypes.c_uint16)
SDL_FUNC("SDLNet_SendDatagram", ctypes.c_bool, ctypes.POINTER(SDLNet_DatagramSocket), ctypes.POINTER(SDLNet_Address), ctypes.c_uint16, ctypes.c_void_p, ctypes.c_int)
SDL_FUNC("SDLNet_ReceiveDatagram", ctypes.c_bool, ctypes.POINTER(SDLNet_DatagramSocket), ctypes.POINTER(ctypes.POINTER(SDLNet_Datagram)))
SDL_FUNC("SDLNet_DestroyDatagram", None, ctypes.POINTER(SDLNet_Datagram))
SDL_FUNC("SDLNet_SimulateDatagramPacketLoss", None, ctypes.POINTER(SDLNet_DatagramSocket), ctypes.c_int)
SDL_FUNC("SDLNet_DestroyDatagramSocket", None, ctypes.POINTER(SDLNet_DatagramSocket))
SDL_FUNC("SDLNet_WaitUntilInputAvailable", ctypes.c_int, ctypes.POINTER(ctypes.c_void_p), ctypes.c_int, ctypes.c_int32)


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_DLL, SDL_GET_DLL, SDL_DLL

SDL_SET_CURRENT_DLL(SDL_DLL)

SDL_SpinLock = ctypes.c_int

SDL_FUNC("SDL_TryLockSpinlock", ctypes.c_bool, ctypes.POINTER(SDL_SpinLock))
SDL_FUNC("SDL_LockSpinlock", None, ctypes.POINTER(SDL_SpinLock))
SDL_FUNC("SDL_UnlockSpinlock", None, ctypes.POINTER(SDL_SpinLock))

SDL_FUNC("SDL_MemoryBarrierReleaseFunction", None)
SDL_FUNC("SDL_MemoryBarrierAcquireFunction", None)

class SDL_AtomicInt(ctypes.Structure):
    _fields_ = [
        ("value", ctypes.c_int)
    ]

SDL_FUNC("SDL_CompareAndSwapAtomicInt", ctypes.c_bool, ctypes.POINTER(SDL_AtomicInt), ctypes.c_int, ctypes.c_int)
SDL_FUNC("SDL_SetAtomicInt", ctypes.c_int, ctypes.POINTER(SDL_AtomicInt), ctypes.c_int)
SDL_FUNC("SDL_GetAtomicInt", ctypes.c_int, ctypes.POINTER(SDL_AtomicInt))
SDL_FUNC("SDL_AddAtomicInt", ctypes.c_int, ctypes.POINTER(SDL_AtomicInt), ctypes.c_int)

class LP_SDL_AtomicInt(ctypes._Pointer):
    ...

def SDL_AtomicIncRef(a: LP_SDL_AtomicInt) -> ctypes.c_int:
    return SDL_GET_DLL(SDL_DLL).SDL_AddAtomicInt(a, 1)

def SDL_AtomicDecRef(a: LP_SDL_AtomicInt) -> ctypes.c_int:
    return SDL_GET_DLL(SDL_DLL).SDL_AddAtomicInt(a, -1) == 1

class SDL_AtomicU32(ctypes.Structure):
    _fields_ = [
        ("value", ctypes.c_uint32)
    ]

SDL_FUNC("SDL_CompareAndSwapAtomicU32", ctypes.c_bool, ctypes.POINTER(SDL_AtomicU32), ctypes.c_uint32, ctypes.c_uint32)
SDL_FUNC("SDL_SetAtomicU32", ctypes.c_uint32, ctypes.POINTER(SDL_AtomicU32), ctypes.c_uint32)
SDL_FUNC("SDL_GetAtomicU32", ctypes.c_uint32, ctypes.POINTER(SDL_AtomicU32))

SDL_FUNC("SDL_CompareAndSwapAtomicPointer", ctypes.c_bool, ctypes.POINTER(ctypes.c_void_p), ctypes.c_void_p, ctypes.c_void_p)
SDL_FUNC("SDL_SetAtomicPointer", ctypes.c_void_p, ctypes.POINTER(ctypes.c_void_p), ctypes.c_void_p)
SDL_FUNC("SDL_GetAtomicPointer", ctypes.c_void_p, ctypes.POINTER(ctypes.c_void_p))


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_DLL, SDL_DLL

from .SDL_video import SDL_Window

SDL_SET_CURRENT_DLL(SDL_DLL)

SDL_MetalView = ctypes.c_void_p

SDL_FUNC("SDL_Metal_CreateView", SDL_MetalView, ctypes.POINTER(SDL_Window))
SDL_FUNC("SDL_Metal_DestroyView", None, SDL_MetalView)
SDL_FUNC("SDL_Metal_GetLayer", ctypes.c_void_p, SDL_MetalView)